package com.diagbot.facade;

import com.diagbot.client.UserServiceClient;
import com.diagbot.dto.ProductAuthProgressDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.UserOrgDTO;
import com.diagbot.dto.WaitExamOrderCouDTO;
import com.diagbot.entity.LantoneProduct;
import com.diagbot.entity.OrderDetails;
import com.diagbot.entity.ProductOrder;
import com.diagbot.entity.ProductOrderIndex;
import com.diagbot.entity.wrapper.ProductOrderWrapper;
import com.diagbot.enums.VisibleIdTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.idc.VisibleIdCreater;
import com.diagbot.service.impl.ProductOrderServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.UserUtils;
import com.diagbot.vo.AddOrderVO;
import com.diagbot.vo.ProductOrderVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 订单业务
 * @author Administrator
 *
 */
@Component
public class ProductOrderFacade extends ProductOrderServiceImpl {
	@Autowired
	private VisibleIdCreater visibleIdCreater;
	@Autowired
	private OrderDetailsFacade orderDetailsFacade;
	@Autowired
	private ServiceInfoFacade serviceInfoFacade;
	@Autowired
	private LantoneProductFacade lantoneProductFacade;
	@Autowired
	private ProductServiceFacade productServiceFacade;
	@Autowired
	private UserServiceClient userServiceClient;
	@Autowired
	private ProductOrderFacade productOrderFacade;
	@Autowired
	private OpenedProductsFacade openedProductsFacade;

	/**
	 * 获取产品的认证进度
	 * @return 包含产品当前的认证状态
	 */
	public List<ProductAuthProgressDTO> productAuthProgress(){
		List<ProductAuthProgressDTO> retList = new ArrayList<ProductAuthProgressDTO>();
		Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
		List<ProductOrder> orderList = baseMapper.getUserProductOrder(userId);
		for(ProductOrder i : orderList){
			ProductAuthProgressDTO productAuthProgressDTO = new ProductAuthProgressDTO();
			productAuthProgressDTO.setNum(i.getNum());
			productAuthProgressDTO.setProducts(baseMapper.getOrderDetails(i.getNum()));
			retList.add(productAuthProgressDTO);
		}

		return retList;
	}

	/**
	 * @Description: 根据用户选择的产品生成订单
	 * @Author: wangyu
	 * @Date: 19:59 2018/9/18
	 */
	@Transactional
	public Boolean addOrders(AddOrderVO addOrderVO){
		Long userId = Long.parseLong(UserUtils.getCurrentPrincipleID());
		ProductOrder productOrder =new ProductOrder();
		productOrder.setCreator(UserUtils.getCurrentPrincipleID());
		productOrder.setGmtCreate(DateUtil.now());
		String orderNum = visibleIdCreater.getNextId(VisibleIdTypeEnum.IS_ORDER.getKey()).toString();
		productOrder.setNum(orderNum);
		productOrder.setUserId(userId);
		productOrder.setTime(DateUtil.now());
		if(!save(productOrder)){
			throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,"添加订单失败");
		}
		for (Long o:addOrderVO.getProductId()) {
			LantoneProduct lantoneProduct = lantoneProductFacade.selectLanProduct(o);
			if(null==lantoneProduct){
				throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,"获取产品信息失败:"+o);
			}
			OrderDetails orderDetails =new OrderDetails();
			orderDetails.setUserId(userId);
			orderDetails.setOrderNum(orderNum);
			orderDetails.setGmtCreate(DateUtil.now());
			orderDetails.setCreator(UserUtils.getCurrentPrincipleID());
			orderDetails.setProductId(o);
			orderDetailsFacade.save(orderDetails);
		}
		return true;
	}

	/**
	 * @Description:  根据订单号查询订单信息
	 * @Author: wangyu
	 * @Date: 19:42 2018/9/20
	 * @param productOrder
	 * @return
	 */
	public ProductOrder selectByOrderNum(ProductOrder productOrder){
		return selctOrderTimeByOrderNum(productOrder);
	}
	
	/**
	 * 计算待审核的订单数量
	 * @return 待审核的订单数量
	 */
    public WaitExamOrderCouDTO waitExamOrderCou(){
    	WaitExamOrderCouDTO waitExamOrderCouDTO = new WaitExamOrderCouDTO();
    	waitExamOrderCouDTO.setCount(baseMapper.waitExamOrderCou());
    	return waitExamOrderCouDTO;
    }

	/**
	 * @Description: 产品申请查询所有订单信息
	 * @Author: wangyu
	 * @Date: 20:23 2018/9/24
	 */
	public List<ProductOrderWrapper> selectAllProductOrder(ProductOrderVO productOrderVO){
		ProductOrderIndex productOrderIndex =new ProductOrderIndex();
		Long curren = productOrderVO.getCurrent();
		Long size =productOrderVO.getSize();
		if(curren==null){
			productOrderVO.setCurrent(1L);
		}
		if (size==null){
			productOrderVO.setSize(10L);
		}
		BeanUtil.copyProperties(productOrderVO, productOrderIndex);
		productOrderIndex.setIndex((productOrderVO.getCurrent().longValue()-1)*productOrderVO.getSize().longValue());
		//TODO
		List<ProductOrderWrapper> list =productOrderFacade.selectAllProductOrder(productOrderIndex);
		List<Long> list1 =new ArrayList<>();
		for (ProductOrderWrapper productOrderWrapper:list) {
			Long longs = productOrderWrapper.getUserId();
			list1.add(longs);
		}
		RespDTO<Map<Long, UserOrgDTO>> mapRespDTO = userServiceClient.getUserAndOrg(list1);
		if(mapRespDTO == null || !"0".equals(mapRespDTO.code) ) {
			throw new CommonException(CommonErrorCode.SERVER_IS_ERROR,
					"获取用户和机构信息失败");
		}
		Map<Long, UserOrgDTO> dataMap = mapRespDTO.data;
		for(ProductOrderWrapper bean : list) {
			UserOrgDTO uo = dataMap.get(bean.getUserId());
			if(uo != null) {
				bean.setUsername(uo.getUsername());
				bean.setLinkman(uo.getLinkman());
				bean.setEmail(uo.getEmail());
				bean.setOrgId(uo.getOrgId());
				bean.setOrgName(uo.getOrgName());
				bean.setIsReject(uo.getIsReject());
				bean.setAuStatus(uo.getAuStatus());
			}
		}
		return list;
	}
}


